@@ -42,25 +42,60 @@ const cmEditor = new CodeMirror(document.getElementById('userFilters'), {
     lineWrapping: true,
     matchBrackets: true,
     maxScanLines: 1,
-    styleActiveLine: true,
+    styleActiveLine: {
+        nonEmpty: true,
+    },
 });
 
 uBlockDashboard.patchCodeMirrorEditor(cmEditor);
 
-vAPI.messaging.send('dashboard', {
-    what: 'getAutoCompleteDetails'
-}).then(response => {
-    if ( response instanceof Object === false ) { return; }
-    const mode = cmEditor.getMode();
-    if ( mode.setHints instanceof Function ) {
-        mode.setHints(response);
-    }
-});
-
 let cachedUserFilters = '';
 
 /******************************************************************************/
 
+// Add auto-complete ability to the editor.
+
+{
+    let hintUpdateToken = 0;
+
+    const responseHandler = function(response) {
+        if ( response instanceof Object === false ) { return; }
+        if ( response.hintUpdateToken !== undefined ) {
+            const mode = cmEditor.getMode();
+            if ( mode.setHints instanceof Function ) {
+                mode.setHints(response);
+            }
+            if ( hintUpdateToken === 0 ) {
+                mode.parser.expertMode = response.expertMode !== false;
+            }
+            hintUpdateToken = response.hintUpdateToken;
+        }
+        vAPI.setTimeout(getHints, 2503);
+    };
+
+    const getHints = function() {
+        vAPI.messaging.send('dashboard', {
+            what: 'getAutoCompleteDetails',
+            hintUpdateToken
+        }).then(responseHandler);
+    };
+
+    getHints();
+}
+
+/******************************************************************************/
+
+const getEditorText = function() {
+    const text = cmEditor.getValue().replace(/\s+$/, '');
+    return text === '' ? text : text + '\n';
+};
+
+const setEditorText = function(text) {
+    cmEditor.setValue(text.replace(/\s+$/, '') + '\n\n');
+};
+
+/******************************************************************************/
+
 // This is to give a visual hint that the content of user blacklist has changed.
 
 const userFiltersChanged = function(changed) {
@@ -81,10 +116,7 @@ const renderUserFilters = async function() {
 
     let content = details.content.trim();
     cachedUserFilters = content;
-    if ( content.length !== 0 ) {
-        content += '\n';
-    }
-    cmEditor.setValue(content);
+    setEditorText(content);
 
     userFiltersChanged(false);
 };
@@ -115,13 +147,10 @@ const handleImportFilePicker = function() {
 
     const fileReaderOnLoadHandler = function() {
         let content = abpImporter(this.result);
-        content = uBlockDashboard.mergeNewLines(
-            cmEditor.getValue().trim(),
-            content
-        );
+        content = uBlockDashboard.mergeNewLines(getEditorText(), content);
         cmEditor.operation(( ) => {
             const cmPos = cmEditor.getCursor();
-            cmEditor.setValue(`${content}\n`);
+            setEditorText(content);
             cmEditor.setCursor(cmPos);
             cmEditor.focus();
         });
@@ -148,7 +177,7 @@ const startImportFilePicker = function() {
 /******************************************************************************/
 
 const exportUserFiltersToFile = function() {
-    const val = cmEditor.getValue().trim();
+    const val = getEditorText();
     if ( val === '' ) { return; }
     const filename = vAPI.i18n('1pExportFilename')
         .replace('{{datetime}}', uBlockDashboard.dateNowToSensibleString())
@@ -164,7 +193,7 @@ const exportUserFiltersToFile = function() {
 const applyChanges = async function() {
     const details = await vAPI.messaging.send('dashboard', {
         what: 'writeUserFilters',
-        content: cmEditor.getValue(),
+        content: getEditorText(),
     });
     if ( details instanceof Object === false || details.error ) { return; }
 
@@ -176,23 +205,19 @@ const applyChanges = async function() {
 };
 
 const revertChanges = function() {
-    let content = cachedUserFilters;
-    if ( content.length !== 0 ) {
-        content += '\n';
-    }
-    cmEditor.setValue(content);
+    setEditorText(cachedUserFilters);
 };
 
 /******************************************************************************/
 
 const getCloudData = function() {
-    return cmEditor.getValue();
+    return getEditorText();
 };
 
 const setCloudData = function(data, append) {
     if ( typeof data !== 'string' ) { return; }
     if ( append ) {
-        data = uBlockDashboard.mergeNewLines(cmEditor.getValue(), data);
+        data = uBlockDashboard.mergeNewLines(getEditorText(), data);
     }
     cmEditor.setValue(data);
 };
@@ -203,7 +228,7 @@ self.cloud.onPull = setCloudData;
 /******************************************************************************/
 
 self.hasUnsavedData = function() {
-    return cmEditor.getValue().trim() !== cachedUserFilters;
+    return getEditorText().trim() !== cachedUserFilters;
 };
 
 /******************************************************************************/
