@@ -477,7 +477,21 @@ const getAssetCacheRegistry = function() {
                 bin instanceof Object &&
                 bin.assetCacheRegistry instanceof Object
             ) {
-                assetCacheRegistry = bin.assetCacheRegistry;
+                if ( Object.keys(assetCacheRegistry).length === 0 ) {
+                    assetCacheRegistry = bin.assetCacheRegistry;
+                } else {
+                    console.error(
+                        'getAssetCacheRegistry(): assetCacheRegistry reassigned!'
+                    );
+                    if (
+                        Object.keys(bin.assetCacheRegistry).sort().join() !==
+                        Object.keys(assetCacheRegistry).sort().join()
+                    ) {
+                        console.error(
+                            'getAssetCacheRegistry(): assetCacheRegistry changes overwritten!'
+                        );
+                    }
+                }
             }
             return assetCacheRegistry;
         });
@@ -538,10 +552,12 @@ const assetCacheRead = async function(assetKey, updateReadTime = false) {
 
 const assetCacheWrite = async function(assetKey, details) {
     let content = '';
+    let options = {};
     if ( typeof details === 'string' ) {
         content = details;
     } else if ( details instanceof Object ) {
         content = details.content || '';
+        options = details;
     }
 
     if ( content === '' ) {
@@ -555,8 +571,8 @@ const assetCacheWrite = async function(assetKey, details) {
         entry = cacheDict[assetKey] = {};
     }
     entry.writeTime = entry.readTime = Date.now();
-    if ( details instanceof Object && typeof details.url === 'string' ) {
-        entry.remoteURL = details.url;
+    if ( typeof options.url === 'string' ) {
+        entry.remoteURL = options.url;
     }
     µBlock.cacheStorage.set({
         assetCacheRegistry,
@@ -565,7 +581,9 @@ const assetCacheWrite = async function(assetKey, details) {
 
     const result = { assetKey, content };
     // https://github.com/uBlockOrigin/uBlock-issues/issues/248
-    fireNotification('after-asset-updated', result);
+    if ( options.silent !== true ) {
+        fireNotification('after-asset-updated', result);
+    }
     return result;
 };
 
@@ -585,14 +603,15 @@ const assetCacheRemove = async function(pattern) {
         delete cacheDict[assetKey];
     }
     if ( removedContent.length !== 0 ) {
-        µBlock.cacheStorage.remove(removedContent);
-        µBlock.cacheStorage.set({ assetCacheRegistry });
+        await Promise.all([
+            µBlock.cacheStorage.remove(removedContent),
+            µBlock.cacheStorage.set({ assetCacheRegistry }),
+        ]);
     }
     for ( let i = 0; i < removedEntries.length; i++ ) {
-        fireNotification(
-            'after-asset-updated',
-            { assetKey: removedEntries[i] }
-        );
+        fireNotification('after-asset-updated', {
+            assetKey: removedEntries[i]
+        });
     }
 };
 
@@ -733,6 +752,7 @@ api.get = async function(assetKey, options = {}) {
             assetCacheWrite(assetKey, {
                 content: details.content,
                 url: contentURL,
+                silent: options.silent === true,
             });
         }
         return reportBack(details.content, contentURL);
