@@ -34,6 +34,8 @@ const redirectNames = new Map();
 const scriptletNames = new Map();
 const preparseDirectiveTokens = new Map();
 const preparseDirectiveHints = [];
+const originHints = [];
+let hintHelperRegistered = false;
 
 /******************************************************************************/
 
@@ -44,22 +46,33 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
     if ( StaticFilteringParser instanceof Object === false ) { return; }
     const parser = new StaticFilteringParser({ interactive: true });
 
+    const reURL = /\bhttps?:\/\/\S+/;
     const rePreparseDirectives = /^!#(?:if|endif|include )\b/;
     const rePreparseIfDirective = /^(!#if ?)(.*)$/;
     let parserSlot = 0;
     let netOptionValueMode = false;
 
     const colorCommentSpan = function(stream) {
-        if ( rePreparseDirectives.test(stream.string) === false ) {
+        const { string, pos } = stream;
+        if ( rePreparseDirectives.test(string) === false ) {
+            const match = reURL.exec(string.slice(pos));
+            if ( match !== null ) {
+                if ( match.index === 0 ) {
+                    stream.pos += match[0].length;
+                    return 'comment link';
+                }
+                stream.pos += match.index;
+                return 'comment';
+            }
             stream.skipToEnd();
             return 'comment';
         }
-        const match = rePreparseIfDirective.exec(stream.string);
+        const match = rePreparseIfDirective.exec(string);
         if ( match === null ) {
             stream.skipToEnd();
             return 'variable strong';
         }
-        if ( stream.pos < match[1].length ) {
+        if ( pos < match[1].length ) {
             stream.pos += match[1].length;
             return 'variable strong';
         }
@@ -95,19 +108,29 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
     };
 
     const colorExtScriptletPatternSpan = function(stream) {
+        const { pos, string } = stream;
         const { i, len } = parser.patternSpan;
-        if ( stream.pos === parser.slices[i+1] ) {
-            stream.pos += 4;
+        const patternBeg = parser.slices[i+1];
+        if ( pos === patternBeg ) {
+            stream.pos = pos + 4;
             return 'def';
         }
         if ( len > 3 ) {
+            if ( pos === patternBeg + 4 ) {
+                const match = /^[^,)]+/.exec(string.slice(pos));
+                const token = match && match[0].trim();
+                if ( token && scriptletNames.has(token) === false ) {
+                    stream.pos = pos + match[0].length;
+                    return 'warning';
+                }
+            }
             const r = parser.slices[i+len+1] - 1;
-            if ( stream.pos < r ) {
+            if ( pos < r ) {
                 stream.pos = r;
                 return 'variable';
             }
-            if ( stream.pos === r ) {
-                stream.pos += 1;
+            if ( pos === r ) {
+                stream.pos = pos + 1;
                 return 'def';
             }
         }
@@ -156,6 +179,70 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
         return null;
     };
 
+    const colorNetOptionValueSpan = function(stream, bits) {
+        const { pos, string } = stream;
+        let style;
+        // Warn about unknown redirect tokens.
+        if (
+            string.charCodeAt(pos - 1) === 0x3D /* '=' */ &&
+            /[$,](redirect(-rule)?|rewrite)=$/.test(string.slice(0, pos))
+        ) {
+            style = 'value';
+            const end = parser.skipUntil(
+                parserSlot,
+                parser.commentSpan.i,
+                parser.BITComma
+            );
+            const raw = parser.strFromSlices(parserSlot, end - 3);
+            const { token } = StaticFilteringParser.parseRedirectValue(raw);
+            if ( redirectNames.has(token) === false ) {
+                style += ' warning';
+            }
+            stream.pos += raw.length;
+            parserSlot = end;
+            return style;
+        }
+        if ( (bits & parser.BITTilde) !== 0 ) {
+            style = 'keyword strong';
+        } else if ( (bits & parser.BITPipe) !== 0 ) {
+            style = 'def';
+        }
+        stream.pos += parser.slices[parserSlot+2];
+        parserSlot += 3;
+        return style || 'value';
+    };
+
+    const colorNetOptionSpan = function(stream) {
+        const bits = parser.slices[parserSlot];
+        if ( (bits & parser.BITComma) !== 0  ) {
+            netOptionValueMode = false;
+            stream.pos += parser.slices[parserSlot+2];
+            parserSlot += 3;
+            return 'def strong';
+        }
+        if ( netOptionValueMode ) {
+            return colorNetOptionValueSpan(stream, bits);
+        }
+        if ( (bits & parser.BITTilde) !== 0 ) {
+            stream.pos += parser.slices[parserSlot+2];
+            parserSlot += 3;
+            return 'keyword strong';
+        }
+        if ( (bits & parser.BITEqual) !== 0 ) {
+            netOptionValueMode = true;
+            stream.pos += parser.slices[parserSlot+2];
+            parserSlot += 3;
+            return 'def';
+        }
+        parserSlot = parser.skipUntil(
+            parserSlot,
+            parser.commentSpan.i,
+            parser.BITComma | parser.BITEqual
+        );
+        stream.pos = parser.slices[parserSlot+1];
+        return 'def';
+    };
+
     const colorNetSpan = function(stream) {
         if ( parserSlot < parser.exceptionSpan.i ) {
             stream.pos += parser.slices[parserSlot+2];
@@ -187,7 +274,9 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
             if ( parser.patternIsRegex() ) {
                 stream.pos = parser.slices[parser.optionsAnchorSpan.i+1];
                 parserSlot = parser.optionsAnchorSpan.i;
-                return 'variable regex';
+                return parser.patternIsTokenizable()
+                    ? 'variable notice'
+                    : 'variable warning';
             }
             if ( (parser.slices[parserSlot] & (parser.BITAsterisk | parser.BITCaret)) !== 0 ) {
                 stream.pos += parser.slices[parserSlot+2];
@@ -215,23 +304,7 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
             parserSlot >= parser.optionsSpan.i &&
             parserSlot < parser.commentSpan.i
         ) {
-            const bits = parser.slices[parserSlot];
-            let style;
-            if ( (bits & parser.BITComma) !== 0  ) {
-                style = 'def strong';
-                netOptionValueMode = false;
-            } else if ( (bits & parser.BITTilde) !== 0 ) {
-                style = 'keyword strong';
-            } else if ( (bits & parser.BITPipe) !== 0 ) {
-                style = 'def';
-            } else if ( netOptionValueMode ) {
-                style = 'value';
-            } else if ( (bits & parser.BITEqual) !== 0 ) {
-                netOptionValueMode = true;
-            }
-            stream.pos += parser.slices[parserSlot+2];
-            parserSlot += 3;
-            return style || 'def';
+            return colorNetOptionSpan(stream);
         }
         if (
             parserSlot >= parser.commentSpan.i &&
@@ -263,10 +336,20 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
             return 'comment';
         }
         if ( parser.category === parser.CATStaticExtFilter ) {
-            return colorExtSpan(stream);
+            const style = colorExtSpan(stream) || '';
+            let flavor = '';
+            if ( (parser.flavorBits & parser.BITFlavorExtCosmetic) !== 0 ) {
+                flavor = 'line-cm-ext-dom';
+            } else if ( (parser.flavorBits & parser.BITFlavorExtScriptlet) !== 0 ) {
+                flavor = 'line-cm-ext-js';
+            } else if ( (parser.flavorBits & parser.BITFlavorExtHTML) !== 0 ) {
+                flavor = 'line-cm-ext-html';
+            }
+            return `${flavor} ${style}`.trim();
         }
         if ( parser.category === parser.CATStaticNetFilter ) {
-            return colorNetSpan(stream);
+            const style = colorNetSpan(stream);
+            return style ? `line-cm-net ${style}` : 'line-cm-net';
         }
         stream.skipToEnd();
         return null;
@@ -275,13 +358,14 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
     return {
         lineComment: '!',
         token: function(stream) {
+            let style = '';
             if ( stream.sol() ) {
                 parser.analyze(stream.string);
                 parser.analyzeExtra();
                 parserSlot = 0;
                 netOptionValueMode = false;
             }
-            let style = colorSpan(stream) || '';
+            style += colorSpan(stream) || '';
             if ( (parser.flavorBits & parser.BITFlavorError) !== 0 ) {
                 style += ' line-background-error';
             }
@@ -289,22 +373,40 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
             return style !== '' ? style : null;
         },
         setHints: function(details) {
-            for ( const [ name, desc ] of details.redirectResources ) {
-                const displayText = desc.aliasOf !== ''
-                    ? `${name} (${desc.aliasOf})`
-                    : '';
-                if ( desc.canRedirect ) {
-                    redirectNames.set(name, displayText);
-                }
-                if ( desc.canInject && name.endsWith('.js') ) {
-                    scriptletNames.set(name.slice(0, -3), displayText);
+            if ( Array.isArray(details.redirectResources) ) {
+                for ( const [ name, desc ] of details.redirectResources ) {
+                    const displayText = desc.aliasOf !== ''
+                        ? `${name} (${desc.aliasOf})`
+                        : '';
+                    if ( desc.canRedirect ) {
+                        redirectNames.set(name, displayText);
+                    }
+                    if ( desc.canInject && name.endsWith('.js') ) {
+                        scriptletNames.set(name.slice(0, -3), displayText);
+                    }
                 }
             }
-            details.preparseDirectiveTokens.forEach(([ a, b ]) => {
-                preparseDirectiveTokens.set(a, b);
-            });
-            preparseDirectiveHints.push(...details.preparseDirectiveHints);
-            initHints();
+            if ( Array.isArray(details.preparseDirectiveTokens)) {
+                details.preparseDirectiveTokens.forEach(([ a, b ]) => {
+                    preparseDirectiveTokens.set(a, b);
+                });
+            }
+            if ( Array.isArray(details.preparseDirectiveHints)) {
+                preparseDirectiveHints.push(...details.preparseDirectiveHints);
+            }
+            if ( Array.isArray(details.originHints) ) {
+                originHints.length = 0;
+                for ( const hint of details.originHints ) {
+                    originHints.push(hint);
+                }
+            }
+            if ( hintHelperRegistered === false ) {
+                hintHelperRegistered = true;
+                initHints();
+            }
+        },
+        get parser() {
+            return parser;
         },
     };
 });
@@ -326,6 +428,12 @@ const initHints = function() {
             return (item[1] & 0b01) !== 0;
         })
     );
+    const excludedHints = new Set([
+        'genericblock',
+        'object-subrequest',
+        'rewrite',
+        'webrtc',
+    ]);
 
     const pickBestHints = function(cursor, seedLeft, seedRight, hints) {
         const seed = (seedLeft + seedRight).trim();
@@ -357,12 +465,39 @@ const initHints = function() {
         };
     };
 
-    const getNetOptionHints = function(cursor, isNegated, seedLeft, seedRight) {
+    const getOriginHints = function(cursor, line, suffix = '') {
+        const beg = cursor.ch;
+        const matchLeft = /[^,|=~]*$/.exec(line.slice(0, beg));
+        const matchRight = /^[^#,|]*/.exec(line.slice(beg));
+        if ( matchLeft === null || matchRight === null ) { return; }
+        const hints = [];
+        for ( const text of originHints ) {
+            hints.push(text + suffix);
+        }
+        return pickBestHints(cursor, matchLeft[0], matchRight[0], hints);
+    };
+
+    const getNetPatternHints = function(cursor, line) {
+        if ( /\|\|[\w.-]*$/.test(line.slice(0, cursor.ch)) ) {
+            return getOriginHints(cursor, line, '^');
+        }
+        // Maybe a static extended filter is meant to be crafted.
+        if ( /[^\w\x80-\xF4#,.-]/.test(line) === false ) {
+            return getOriginHints(cursor, line);
+        }
+    };
+
+    const getNetOptionHints = function(cursor, seedLeft, seedRight) {
+        const isNegated = seedLeft.startsWith('~');
+        if ( isNegated ) {
+            seedLeft = seedLeft.slice(1);
+        }
         const assignPos = seedRight.indexOf('=');
         if ( assignPos !== -1 ) { seedRight = seedRight.slice(0, assignPos); }
         const isException = parser.isException();
         const hints = [];
-        for ( let [ text, bits ] of parser.netOptionTokens ) {
+        for ( let [ text, bits ] of parser.netOptionTokenDescriptors ) {
+            if ( excludedHints.has(text) ) { continue; }
             if ( isNegated && (bits & parser.OPTCanNegate) === 0 ) { continue; }
             if ( isException ) {
                 if ( (bits & parser.OPTBlockOnly) !== 0 ) { continue; }
@@ -380,6 +515,7 @@ const initHints = function() {
     const getNetRedirectHints = function(cursor, seedLeft, seedRight) {
         const hints = [];
         for ( const text of redirectNames.keys() ) {
+            if ( text.startsWith('abp-resource:') ) { continue; }
             hints.push(text);
         }
         return pickBestHints(cursor, seedLeft, seedRight, hints);
@@ -387,26 +523,28 @@ const initHints = function() {
 
     const getNetHints = function(cursor, line) {
         const beg = cursor.ch;
-        if ( beg < parser.optionsSpan ) { return; }
+        if ( beg <= parser.slices[parser.optionsAnchorSpan.i+1] ) {
+            return getNetPatternHints(cursor, line);
+        }
         const lineBefore = line.slice(0, beg);
         const lineAfter = line.slice(beg);
-        let matchLeft = /~?([^$,~]*)$/.exec(lineBefore);
-        let matchRight = /^([^,]*)/.exec(lineAfter);
+        let matchLeft = /[^$,]*$/.exec(lineBefore);
+        let matchRight = /^[^,]*/.exec(lineAfter);
         if ( matchLeft === null || matchRight === null ) { return; }
-        let pos = matchLeft[1].indexOf('=');
-        if ( pos === -1 ) {
-            return getNetOptionHints(
+        const assignPos = matchLeft[0].indexOf('=');
+        if ( assignPos === -1 ) {
+            return getNetOptionHints(cursor, matchLeft[0], matchRight[0]);
+        }
+        if ( /^(redirect(-rule)?|rewrite)=/.test(matchLeft[0]) ) {
+            return getNetRedirectHints(
                 cursor,
-                matchLeft[0].startsWith('~'),
-                matchLeft[1],
-                matchRight[1]
+                matchLeft[0].slice(assignPos + 1),
+                matchRight[0]
             );
         }
-        return getNetRedirectHints(
-            cursor,
-            matchLeft[1].slice(pos + 1),
-            matchRight[1]
-        );
+        if ( matchLeft[0].startsWith('domain=') ) {
+            return getOriginHints(cursor, line);
+        }
     };
 
     const getExtSelectorHints = function(cursor, line) {
@@ -466,10 +604,15 @@ const initHints = function() {
         const line = cm.getLine(cursor.line);
         parser.analyze(line);
         if ( parser.category === parser.CATStaticExtFilter ) {
-            if ( parser.hasFlavor(parser.BITFlavorExtScriptlet) ) {
-                return getExtScriptletHints(cursor, line);
+            let hints;
+            if ( cursor.ch <= parser.slices[parser.optionsAnchorSpan.i+1] ) {
+                hints = getOriginHints(cursor, line);
+            } else {
+                hints = parser.hasFlavor(parser.BITFlavorExtScriptlet)
+                    ? getExtScriptletHints(cursor, line)
+                    : getExtSelectorHints(cursor, line);
             }
-            return getExtSelectorHints(cursor, line);
+            return hints;
         }
         if ( parser.category === parser.CATStaticNetFilter ) {
             return getNetHints(cursor, line);
@@ -477,13 +620,15 @@ const initHints = function() {
         if ( parser.category === parser.CATComment ) {
             return getCommentHints(cursor, line);
         }
+        if ( parser.category === parser.CATNone ) {
+            return getOriginHints(cursor, line);
+        }
     });
 };
 
 /******************************************************************************/
 
 CodeMirror.registerHelper('fold', 'ubo-static-filtering', (( ) => {
-
     const foldIfEndif = function(startLineNo, startLine, cm) {
         const lastLineNo = cm.lastLine();
         let endLineNo = startLineNo;
@@ -539,6 +684,94 @@ CodeMirror.registerHelper('fold', 'ubo-static-filtering', (( ) => {
 
 /******************************************************************************/
 
+// Enhanced word selection
+
+{
+    const selectWordAt = function(cm, pos) {
+        const { line, ch } = pos;
+        const s = cm.getLine(line);
+        const { type: token } = cm.getTokenAt(pos);
+        let beg, end;
+
+        // Select URL in comments
+        if ( /\bcomment\b/.test(token) && /\blink\b/.test(token) ) {
+            const l = /\S+$/.exec(s.slice(0, ch));
+            if ( l && /^https?:\/\//.test(s.slice(l.index)) ) {
+                const r = /^\S+/.exec(s.slice(ch));
+                if ( r ) {
+                    beg = l.index;
+                    end = ch + r[0].length;
+                }
+            }
+        }
+
+        // Better word selection for extended filters: prefix
+        else if (
+            /\bline-cm-ext-(?:dom|html|js)\b/.test(token) &&
+            /\bvalue\b/.test(token)
+        ) {
+            const l = /[^,.]*$/i.exec(s.slice(0, ch));
+            const r = /^[^#,]*/i.exec(s.slice(ch));
+            if ( l && r ) {
+                beg = l.index;
+                end = ch + r[0].length;
+            }
+        }
+
+        // Better word selection for cosmetic and HTML filters: suffix
+        else if ( /\bline-cm-ext-(?:dom|html)\b/.test(token) ) {
+            const l = /[#.]?[a-z0-9_-]+$/i.exec(s.slice(0, ch));
+            const r = /^[a-z0-9_-]+/i.exec(s.slice(ch));
+            if ( l && r ) {
+                beg = l.index;
+                end = ch + r[0].length;
+                if ( /\bdef\b/.test(cm.getTokenTypeAt({ line, ch: beg + 1 })) ) {
+                    beg += 1;
+                }
+            }
+        }
+
+        // Better word selection for network filters
+        else if ( /\bline-cm-net\b/.test(token) ) {
+            if ( /\bvalue\b/.test(token) ) {
+                const l = /[^ ,.=|]*$/i.exec(s.slice(0, ch));
+                const r = /^[^ #,|]*/i.exec(s.slice(ch));
+                if ( l && r ) {
+                    beg = l.index;
+                    end = ch + r[0].length;
+                }
+            } else if ( /\bdef\b/.test(token) ) {
+                const l = /[a-z0-9-]+$/i.exec(s.slice(0, ch));
+                const r = /^[^,]*=[^,]+/i.exec(s.slice(ch));
+                if ( l && r ) {
+                    beg = l.index;
+                    end = ch + r[0].length;
+                }
+            }
+        }
+
+        if ( beg === undefined ) {
+            const { anchor, head } = cm.findWordAt(pos);
+            return { from: anchor, to: head };
+        }
+
+        return {
+            from: { line, ch: beg },
+            to: { line, ch: end },
+        };
+    };
+
+    CodeMirror.defineInitHook(cm => {
+        cm.setOption('configureMouse', function(cm, repeat) {
+            return {
+                unit: repeat === 'double' ? selectWordAt : null,
+            };
+        });
+    });
+}
+
+/******************************************************************************/
+
 // <<<<< end of local scope
 }
 
