@@ -116,7 +116,8 @@ vAPI.contentScript = true;
     try {
         while (
             context !== self.top &&
-            context.location.protocol === 'about:'
+            context.location.href.startsWith('about:blank') &&
+            context.parent.location.href
         ) {
             context = context.parent;
         }
@@ -362,7 +363,7 @@ vAPI.SafeAnimationFrame = class {
     };
 
     // https://github.com/chrisaljoudi/uBlock/issues/205
-    // Do not handle added node directly from within mutation observer.
+    //   Do not handle added node directly from within mutation observer.
     const observerHandler = function(mutations) {
         let i = mutations.length;
         while ( i-- ) {
@@ -376,7 +377,7 @@ vAPI.SafeAnimationFrame = class {
                 removedNodeLists.push(nodeList);
             }
         }
-        if ( addedNodeLists.length !== 0 || removedNodes ) {
+        if ( addedNodeLists.length !== 0 || removedNodeLists.length !== 0 ) {
             safeObserverHandlerTimer.start(
                 addedNodeLists.length < 100 ? 1 : undefined
             );
@@ -433,7 +434,7 @@ vAPI.SafeAnimationFrame = class {
     const cleanup = function() {
         if ( domLayoutObserver !== undefined ) {
             domLayoutObserver.disconnect();
-            domLayoutObserver = null;
+            domLayoutObserver = undefined;
         }
         if ( safeObserverHandlerTimer !== undefined ) {
             safeObserverHandlerTimer.clear();
@@ -851,7 +852,7 @@ vAPI.injectScriptlet = function(doc, text) {
             this.domFilterer.addCSSRule(
                 `[${this.masterToken}][${styleToken}]`,
                 style,
-                { silent: true }
+                { silent: true, mustInject: true }
             );
             return styleToken;
         }
@@ -867,6 +868,7 @@ vAPI.injectScriptlet = function(doc, text) {
             for ( const node of nodes ) {
                 node.setAttribute(this.masterToken, '');
                 node.setAttribute(styleToken, '');
+                this.styledNodes.add(node);
             }
         }
 
@@ -908,7 +910,6 @@ vAPI.injectScriptlet = function(doc, text) {
             this.disabled = false;
             this.listeners = [];
             this.filterset = new Set();
-            this.addedCSSRules = new Set();
             this.exceptedCSSRules = [];
             this.exceptions = [];
             this.proceduralFilterer = null;
@@ -929,22 +930,11 @@ vAPI.injectScriptlet = function(doc, text) {
         addCSSRule(selectors, declarations, details = {}) {
             if ( selectors === undefined ) { return; }
             const selectorsStr = Array.isArray(selectors)
-                    ? selectors.join(',\n')
-                    : selectors;
+                ? selectors.join(',\n')
+                : selectors;
             if ( selectorsStr.length === 0 ) { return; }
-            const entry = {
-                selectors: selectorsStr,
-                declarations,
-                lazy: details.lazy === true,
-                injected: details.injected === true
-            };
-            this.addedCSSRules.add(entry);
-            this.filterset.add(entry);
-            if (
-                this.disabled === false &&
-                entry.lazy !== true &&
-                entry.injected !== true
-            ) {
+            this.filterset.add({ selectors: selectorsStr, declarations });
+            if ( details.mustInject && this.disabled === false ) {
                 vAPI.userStylesheet.add(`${selectorsStr}\n{${declarations}}`);
             }
             this.commit();
@@ -1012,20 +1002,7 @@ vAPI.injectScriptlet = function(doc, text) {
         commitNow() {
             this.commitTimer.clear();
             if ( vAPI instanceof Object === false ) { return; }
-            const userStylesheet = vAPI.userStylesheet;
-            for ( const entry of this.addedCSSRules ) {
-                if (
-                    this.disabled === false &&
-                    entry.lazy &&
-                    entry.injected === false
-                ) {
-                    userStylesheet.add(
-                        `${entry.selectors}\n{${entry.declarations}}`
-                    );
-                }
-            }
-            this.addedCSSRules.clear();
-            userStylesheet.apply();
+            vAPI.userStylesheet.apply();
             if ( this.proceduralFilterer instanceof Object ) {
                 this.proceduralFilterer.commitNow();
             }
@@ -1059,11 +1036,11 @@ vAPI.injectScriptlet = function(doc, text) {
                     o.action[0] === ':style' &&
                     o.tasks === undefined
                 ) {
-                    this.addCSSRule(o.selector, o.action[1]);
+                    this.addCSSRule(o.selector, o.action[1], { mustInject: true });
                     continue;
                 }
                 if ( o.pseudo !== undefined ) {
-                    this.addCSSRule(o.selector, vAPI.hideStyle);
+                    this.addCSSRule(o.selector, vAPI.hideStyle, { mustInject: true });
                     continue;
                 }
                 procedurals.push(o);
@@ -1124,21 +1101,26 @@ vAPI.injectScriptlet = function(doc, text) {
     const messaging = vAPI.messaging;
     const toCollapse = new Map();
     const src1stProps = {
+        audio: 'currentSrc',
         embed: 'src',
         iframe: 'src',
-        img: 'src',
-        object: 'data'
+        img: 'currentSrc',
+        object: 'data',
+        video: 'currentSrc',
     };
     const src2ndProps = {
-        img: 'srcset'
+        audio: 'src',
+        img: 'src',
+        video: 'src',
     };
     const tagToTypeMap = {
+        audio: 'media',
         embed: 'object',
         iframe: 'sub_frame',
         img: 'image',
-        object: 'object'
+        object: 'object',
+        video: 'media',
     };
-
     let resquestIdGenerator = 1,
         processTimer,
         cachedBlockedSet,
@@ -1154,6 +1136,21 @@ vAPI.injectScriptlet = function(doc, text) {
         cachedBlockedSetTimer = undefined;
     };
 
+    // https://github.com/chrisaljoudi/uBlock/issues/399
+    // https://github.com/gorhill/uBlock/issues/2848
+    //   Use a user stylesheet to collapse placeholders.
+    const getCollapseToken = ( ) => {
+        if ( collapseToken === undefined ) {
+            collapseToken = vAPI.randomToken();
+            vAPI.userStylesheet.add(
+                `[${collapseToken}]\n{display:none!important;}`,
+                true
+            );
+        }
+        return collapseToken;
+    };
+    let collapseToken;
+
     // https://github.com/chrisaljoudi/uBlock/issues/174
     //   Do not remove fragment from src URL
     const onProcessed = function(response) {
@@ -1165,6 +1162,7 @@ vAPI.injectScriptlet = function(doc, text) {
 
         const targets = toCollapse.get(response.id);
         if ( targets === undefined ) { return; }
+
         toCollapse.delete(response.id);
         if ( cachedBlockedSetHash !== response.hash ) {
             cachedBlockedSet = new Set(response.blockedResources);
@@ -1177,8 +1175,8 @@ vAPI.injectScriptlet = function(doc, text) {
         if ( cachedBlockedSet === undefined || cachedBlockedSet.size === 0 ) {
             return;
         }
+
         const selectors = [];
-        const iframeLoadEventPatch = vAPI.iframeLoadEventPatch;
         let netSelectorCacheCountMax = response.netSelectorCacheCountMax;
 
         for ( const target of targets ) {
@@ -1195,32 +1193,24 @@ vAPI.injectScriptlet = function(doc, text) {
             if ( cachedBlockedSet.has(tagToTypeMap[tag] + ' ' + src) === false ) {
                 continue;
             }
-            // https://github.com/chrisaljoudi/uBlock/issues/399
-            // Never remove elements from the DOM, just hide them
-            target.style.setProperty('display', 'none', 'important');
-            target.hidden = true;
+            target.setAttribute(getCollapseToken(), '');
             // https://github.com/chrisaljoudi/uBlock/issues/1048
-            // Use attribute to construct CSS rule
-            if ( netSelectorCacheCount <= netSelectorCacheCountMax ) {
-                const value = target.getAttribute(prop);
-                if ( value ) {
-                    selectors.push(`${tag}[${prop}="${CSS.escape(value)}"]`);
-                    netSelectorCacheCount += 1;
-                }
-            }
-            if ( iframeLoadEventPatch !== undefined ) {
-                iframeLoadEventPatch(target);
+            //   Use attribute to construct CSS rule
+            if ( netSelectorCacheCount > netSelectorCacheCountMax ) { continue; }
+            const value = target.getAttribute(prop);
+            if ( value ) {
+                selectors.push(`${tag}[${prop}="${CSS.escape(value)}"]`);
+                netSelectorCacheCount += 1;
             }
         }
 
-        if ( selectors.length !== 0 ) {
-            messaging.send('contentscript', {
-                what: 'cosmeticFiltersInjected',
-                type: 'net',
-                hostname: window.location.hostname,
-                selectors,
-            });
-        }
+        if ( selectors.length === 0 ) { return; }
+        messaging.send('contentscript', {
+            what: 'cosmeticFiltersInjected',
+            type: 'net',
+            hostname: window.location.hostname,
+            selectors,
+        });
     };
 
     const send = function() {
@@ -1537,11 +1527,7 @@ vAPI.injectScriptlet = function(doc, text) {
         if ( result ) {
             let selectors = result.injected;
             if ( typeof selectors === 'string' && selectors.length !== 0 ) {
-                domFilterer.addCSSRule(
-                    selectors,
-                    vAPI.hideStyle,
-                    { injected: true }
-                );
+                domFilterer.addCSSRule(selectors, vAPI.hideStyle);
                 mustCommit = true;
             }
             selectors = result.excepted;
@@ -1709,11 +1695,7 @@ vAPI.injectScriptlet = function(doc, text) {
                 vAPI.domSurveyor = null;
             }
             domFilterer.exceptions = cfeDetails.exceptionFilters;
-            domFilterer.addCSSRule(
-                cfeDetails.injectedHideFilters,
-                vAPI.hideStyle,
-                { injected: true }
-            );
+            domFilterer.addCSSRule(cfeDetails.injectedHideFilters, vAPI.hideStyle);
             domFilterer.addProceduralSelectors(cfeDetails.proceduralFilters);
             domFilterer.exceptCSSRules(cfeDetails.exceptedFilters);
         }
@@ -1753,7 +1735,6 @@ vAPI.injectScriptlet = function(doc, text) {
         vAPI.messaging.send('contentscript', {
             what: 'retrieveContentScriptParameters',
             url: vAPI.effectiveSelf.location.href,
-            charset: document.characterSet,
         }).then(response => {
             bootstrapPhase1(response);
         });
