@@ -214,6 +214,7 @@ const LogEntry = function(details) {
             this[prop] = details[prop];
         }
     }
+    this.type = details.stype || details.type;
     if ( details.aliasURL !== undefined ) {
         this.aliased = true;
     }
@@ -301,7 +302,13 @@ const processLoggerEntries = function(response) {
             if ( autoDeleteVoidedRows ) { continue; }
             parsed.voided = true;
         }
-        if ( parsed.type === 'main_frame' && parsed.aliased === false ) {
+        if (
+            parsed.type === 'main_frame' &&
+            parsed.aliased === false && (
+                parsed.filter === undefined ||
+                parsed.filter.source !== 'redirect'
+            )
+        ) {
             const separator = createLogSeparator(parsed, unboxed.url);
             loggerEntries.unshift(separator);
             if ( rowFilterer.filterOne(separator) ) {
@@ -639,7 +646,6 @@ const viewPort = (( ) => {
         const divcl = div.classList;
         let span;
 
-
         // Realm
         if ( details.realm !== undefined ) {
             divcl.add(details.realm + 'Realm');
@@ -679,6 +685,9 @@ const viewPort = (( ) => {
             }
             if ( filteringType === 'static' ) {
                 divcl.add('canLookup');
+                if ( filter.modifier === true ) {
+                    div.setAttribute('data-modifier', '');
+                }
             } else if ( filteringType === 'cosmetic' ) {
                 divcl.add('canLookup');
                 divcl.toggle('isException', filter.raw.startsWith('#@#'));
@@ -1263,13 +1272,16 @@ const reloadTab = function(ev) {
             // Avoid duplicates
             if ( createdStaticFilters.hasOwnProperty(value) ) { return; }
             createdStaticFilters[value] = true;
+            // https://github.com/uBlockOrigin/uBlock-issues/issues/1281#issuecomment-704217175
+            // TODO:
+            //   Figure a way to use the actual document URL. Currently using
+            //   a synthetic URL derived from the document hostname.
             if ( value !== '' ) {
                 messaging.send('loggerUI', {
                     what: 'createUserFilter',
                     autoComment: true,
                     filters: value,
-                    origin: targetPageDomain,
-                    pageDomain: targetPageDomain,
+                    docURL: `https://${targetFrameHostname}/`,
                 });
             }
             updateWidgets();
@@ -1641,6 +1653,9 @@ const reloadTab = function(ev) {
             const attr = tr.getAttribute('data-status') || '';
             if ( attr !== '' ) {
                 rows[7].setAttribute('data-status', attr);
+                if ( tr.hasAttribute('data-modifier') ) {
+                    rows[7].setAttribute('data-modifier', '');
+                }
             }
             rows[7].children[1].appendChild(trch[6].cloneNode(true));
         } else {
@@ -1872,8 +1887,6 @@ const reloadTab = function(ev) {
     );
 })();
 
-// https://www.youtube.com/watch?v=XyNYrmmdUd4
-
 /******************************************************************************/
 /******************************************************************************/
 
