@@ -1027,7 +1027,7 @@ uDom('[data-i18n="popupAnyRulePrompt"]').on('click', ev => {
         messaging.send('popupPanel', {
             what: 'gotoURL',
             details: {
-                url: `popup-fenix.html?tabId=${popupData.tabId}`,
+                url: `popup-fenix.html?tabId=${popupData.tabId}&intab=1`,
                 select: true,
                 index: -1,
             },
@@ -1260,6 +1260,9 @@ const getPopupData = async function(tabId) {
                 panes.prepend(sticky);
             }
         }
+        if ( selfURL.searchParams.get('intab') !== null ) {
+            root.classList.add('intab');
+        }
         await nextFrames(1);
         document.body.classList.remove('loading');
     };
