@@ -94,64 +94,6 @@
                     }
                 };
                 injectScriptlets(document);
-                const processIFrame = function(iframe) {
-                    const src = iframe.src;
-                    if ( /^https?:\/\//.test(src) === false ) {
-                        injectScriptlets(iframe.contentDocument);
-                    }
-                };
-                let observerTimer,
-                    observerLists = [];
-                const observerAsync = function() {
-                    for ( const nodelist of observerLists ) {
-                        for ( const node of nodelist ) {
-                            if ( node.nodeType !== 1 ) { continue; }
-                            if ( node.parentElement === null ) { continue; }
-                            if ( node.localName === 'iframe' ) {
-                                processIFrame(node);
-                            }
-                            if ( node.childElementCount === 0 ) { continue; }
-                            let iframes = node.querySelectorAll('iframe');
-                            for ( const iframe of iframes ) {
-                                processIFrame(iframe);
-                            }
-                        }
-                    }
-                    observerLists = [];
-                    observerTimer = undefined;
-                };
-                const ready = function(ev) {
-                    if ( ev !== undefined ) {
-                        window.removeEventListener(ev.type, ready);
-                    }
-                    const iframes = document.getElementsByTagName('iframe');
-                    if ( iframes.length !== 0 ) {
-                        observerLists.push(iframes);
-                        observerTimer = setTimeout(observerAsync, 1);
-                    }
-                    const observer = new MutationObserver(function(mutations) {
-                        for ( const mutation of mutations ) {
-                            if ( mutation.addedNodes.length !== 0 ) {
-                                observerLists.push(mutation.addedNodes);
-                            }
-                        }
-                        if (
-                            observerLists.length !== 0 &&
-                            observerTimer === undefined
-                        ) {
-                            observerTimer = setTimeout(observerAsync, 1);
-                        }
-                    });
-                    observer.observe(
-                        document.documentElement,
-                        { childList: true, subtree: true }
-                    );
-                };
-                if ( document.readyState === 'loading' ) {
-                    window.addEventListener('DOMContentLoaded', ready);
-                } else {
-                    ready();
-                }
             }.toString(),
             ')(',
                 '"', 'hostname-slot', '", ',
@@ -288,8 +230,7 @@
     };
 
     api.compile = function(parser, writer) {
-        // 1001 = scriptlet injection
-        writer.select(1001);
+        writer.select(µb.compiledScriptletSection);
 
         // Only exception filters are allowed to be global.
         const { raw, exception } = parser.result;
@@ -328,8 +269,7 @@
     //     4                -1
 
     api.fromCompiledContent = function(reader) {
-        // 1001 = scriptlet injection
-        reader.select(1001);
+        reader.select(µb.compiledScriptletSection);
 
         while ( reader.next() ) {
             acceptedCount += 1;
@@ -378,12 +318,10 @@
             sessionScriptletDB.retrieve([ null, $exceptions ]);
         }
         scriptletDB.retrieve(hostname, [ $scriptlets, $exceptions ]);
-        if ( request.entity !== '' ) {
-            scriptletDB.retrieve(
-                `${hostname.slice(0, -request.domain.length)}${request.entity}`,
-                [ $scriptlets, $exceptions ]
-            );
-        }
+        const entity = request.entity !== ''
+            ? `${hostname.slice(0, -request.domain.length)}${request.entity}`
+            : '*';
+        scriptletDB.retrieve(entity, [ $scriptlets, $exceptions ], 1);
         if ( $scriptlets.size === 0 ) { return; }
 
         const loggerEnabled = µb.logger.enabled;
@@ -439,7 +377,6 @@
 
     api.injectNow = function(details) {
         if ( typeof details.frameId !== 'number' ) { return; }
-        if ( µb.URI.isNetworkURI(details.url) === false ) { return; }
         const request = {
             tabId: details.tabId,
             frameId: details.frameId,
@@ -457,10 +394,10 @@
             code = 'debugger;\n' + code;
         }
         vAPI.tabs.executeScript(details.tabId, {
-            code: code,
+            code,
             frameId: details.frameId,
-            matchAboutBlank: false,
-            runAt: 'document_start'
+            matchAboutBlank: true,
+            runAt: 'document_start',
         });
     };
 
@@ -490,7 +427,7 @@
         const t0 = self.performance.now();
         for ( let i = 0; i < requests.length; i++ ) {
             const request = requests[i];
-            if ( request.cpt !== 'document' ) { continue; }
+            if ( request.cpt !== 'main_frame' ) { continue; }
             count += 1;
             details.url = request.url;
             details.hostname = µb.URI.hostnameFromURI(request.url);
