@@ -23,170 +23,6 @@
 
 /******************************************************************************/
 
-// A standalone URL tokenizer will allow us to use URL tokens in more than
-// just static filtering engine. This opens the door to optimize other
-// filtering engine parts aside static filtering. This also allows:
-// - Tokenize only on demand.
-// - To potentially avoid tokenizing when same URL is fed to tokenizer.
-//   - Benchmarking shows this to be a common occurrence.
-//
-// https://github.com/gorhill/uBlock/issues/2630
-// Slice input URL into a list of safe-integer token values, instead of a list
-// of substrings. The assumption is that with dealing only with numeric
-// values, less underlying memory allocations, and also as a consequence
-// less work for the garbage collector down the road.
-// Another assumption is that using a numeric-based key value for Map() is
-// more efficient than string-based key value (but that is something I would
-// have to benchmark).
-// Benchmark for string-based tokens vs. safe-integer token values:
-//   https://gorhill.github.io/obj-vs-set-vs-map/tokenize-to-str-vs-to-int.html
-
-µBlock.urlTokenizer = new (class {
-    constructor() {
-        this._chars = '0123456789%abcdefghijklmnopqrstuvwxyz';
-        this._validTokenChars = new Uint8Array(128);
-        for ( let i = 0, n = this._chars.length; i < n; i++ ) {
-            this._validTokenChars[this._chars.charCodeAt(i)] = i + 1;
-        }
-        // Four upper bits of token hash are reserved for built-in predefined
-        // token hashes, which should never end up being used when tokenizing
-        // any arbitrary string.
-             this.dotTokenHash = 0x10000000;
-             this.anyTokenHash = 0x20000000;
-        this.anyHTTPSTokenHash = 0x30000000;
-         this.anyHTTPTokenHash = 0x40000000;
-              this.noTokenHash = 0x50000000;
-           this.emptyTokenHash = 0xF0000000;
-
-        this._urlIn = '';
-        this._urlOut = '';
-        this._tokenized = false;
-        // https://www.reddit.com/r/uBlockOrigin/comments/dzw57l/
-        //   Remember: 1 token needs two slots
-        this._tokens = new Uint32Array(2064);
-
-        this.knownTokens = new Uint8Array(65536);
-        this.resetKnownTokens();
-        this.MAX_TOKEN_LENGTH = 7;
-    }
-
-    setURL(url) {
-        if ( url !== this._urlIn ) {
-            this._urlIn = url;
-            this._urlOut = url.toLowerCase();
-            this._tokenized = false;
-        }
-        return this._urlOut;
-    }
-
-    resetKnownTokens() {
-        this.knownTokens.fill(0);
-        this.addKnownToken(this.dotTokenHash);
-        this.addKnownToken(this.anyTokenHash);
-        this.addKnownToken(this.anyHTTPSTokenHash);
-        this.addKnownToken(this.anyHTTPTokenHash);
-        this.addKnownToken(this.noTokenHash);
-    }
-
-    addKnownToken(th) {
-        this.knownTokens[th & 0xFFFF ^ th >>> 16] = 1;
-    }
-
-    // Tokenize on demand.
-    getTokens(encodeInto) {
-        if ( this._tokenized ) { return this._tokens; }
-        let i = this._tokenize(encodeInto);
-        this._tokens[i+0] = this.anyTokenHash;
-        this._tokens[i+1] = 0;
-        i += 2;
-        if ( this._urlOut.startsWith('https://') ) {
-            this._tokens[i+0] = this.anyHTTPSTokenHash;
-            this._tokens[i+1] = 0;
-            i += 2;
-        } else if ( this._urlOut.startsWith('http://') ) {
-            this._tokens[i+0] = this.anyHTTPTokenHash;
-            this._tokens[i+1] = 0;
-            i += 2;
-        }
-        this._tokens[i+0] = this.noTokenHash;
-        this._tokens[i+1] = 0;
-        this._tokens[i+2] = 0;
-        this._tokenized = true;
-        return this._tokens;
-    }
-
-    tokenHashFromString(s) {
-        const l = s.length;
-        if ( l === 0 ) { return this.emptyTokenHash; }
-        const vtc = this._validTokenChars;
-        let th = vtc[s.charCodeAt(0)];
-        for ( let i = 1; i !== 7 && i !== l; i++ ) {
-            th = th << 4 ^ vtc[s.charCodeAt(i)];
-        }
-        return th;
-    }
-
-    stringFromTokenHash(th) {
-        if ( th === 0 ) { return ''; }
-        return th.toString(16);
-    }
-
-    toSelfie() {
-        return µBlock.base64.encode(
-            this.knownTokens.buffer,
-            this.knownTokens.byteLength
-        );
-    }
-
-    fromSelfie(selfie) {
-        return µBlock.base64.decode(selfie, this.knownTokens.buffer);
-    }
-
-    // https://github.com/chrisaljoudi/uBlock/issues/1118
-    // We limit to a maximum number of tokens.
-
-    _tokenize(encodeInto) {
-        const tokens = this._tokens;
-        let url = this._urlOut;
-        let l = url.length;
-        if ( l === 0 ) { return 0; }
-        if ( l > 2048 ) {
-            url = url.slice(0, 2048);
-            l = 2048;
-        }
-        encodeInto.haystackLen = l;
-        const knownTokens = this.knownTokens;
-        const vtc = this._validTokenChars;
-        const charCodes = encodeInto.haystack;
-        let i = 0, j = 0, n, ti, th;
-        for (;;) {
-            for (;;) {
-                if ( i === l ) { return j; }
-                th = vtc[(charCodes[i] = url.charCodeAt(i))];
-                i += 1;
-                if ( th !== 0 ) { break; }
-            }
-            ti = i - 1; n = 1;
-            for (;;) {
-                if ( i === l ) { break; }
-                const v = vtc[(charCodes[i] = url.charCodeAt(i))];
-                i += 1;
-                if ( v === 0 ) { break; }
-                if ( n === 7 ) { continue; }
-                th = th << 4 ^ v;
-                n += 1;
-            }
-            if ( knownTokens[th & 0xFFFF ^ th >>> 16] !== 0 ) {
-                tokens[j+0] = th;
-                tokens[j+1] = ti;
-                j += 2;
-            }
-        }
-    }
-})();
-
-/******************************************************************************/
-
 µBlock.formatCount = function(count) {
     if ( typeof count !== 'number' ) {
         return '';
@@ -317,6 +153,7 @@
             if ( this.block === undefined ) {
                 this.blocks.set(blockId, (this.block = []));
             }
+            return this;
         }
         toString() {
             let result = [];
@@ -343,7 +180,7 @@
             this.blocks = new Map();
             this.properties = new Map();
             let reBlockStart = new RegExp(
-                '^' + this.io.blockStartPrefix + '(\\d+)\\n',
+                `^${this.io.blockStartPrefix}(\\d+)\\n`,
                 'gm'
             );
             let match = reBlockStart.exec(raw);
@@ -819,6 +656,11 @@
                 }
                 if ( request instanceof Object === false ) { continue; }
                 if ( !request.frameUrl || !request.url ) { continue; }
+                if ( request.cpt === 'document' ) {
+                    request.cpt = 'main_frame';
+                } else if ( request.cpt === 'xhr' ) {
+                    request.cpt = 'xmlhttprequest';
+                }
                 requests.push(request);
             }
             return requests;
@@ -845,15 +687,43 @@
 
 /******************************************************************************/
 
-µBlock.getMessageSenderDetails = function(sender) {
-    const r = {};
-    if ( sender instanceof Object ) {
-        r.url = sender.url;
-        r.frameId = sender.frameId;
-        const tab = sender.tab;
-        if ( tab instanceof Object ) {
-            r.tabId = tab.id;
+µBlock.getModifiedSettings = function(edit, orig = {}) {
+    const out = {};
+    for ( const prop in edit ) {
+        if ( orig.hasOwnProperty(prop) && edit[prop] !== orig[prop] ) {
+            out[prop] = edit[prop];
         }
     }
+    return out;
+};
+
+µBlock.settingValueFromString = function(orig, name, s) {
+    if ( typeof name !== 'string' || typeof s !== 'string' ) { return; }
+    if ( orig.hasOwnProperty(name) === false ) { return; }
+    let r;
+    switch ( typeof orig[name] ) {
+    case 'boolean':
+        if ( s === 'true' ) {
+            r = true;
+        } else if ( s === 'false' ) {
+            r = false;
+        }
+        break;
+    case 'string':
+        r = s.trim();
+        break;
+    case 'number':
+        if ( s.startsWith('0b') ) {
+            r = parseInt(s.slice(2), 2);
+        } else if ( s.startsWith('0x') ) {
+            r = parseInt(s.slice(2), 16);
+        } else {
+            r = parseInt(s, 10);
+        }
+        if ( isNaN(r) ) { r = undefined; }
+        break;
+    default:
+        break;
+    }
     return r;
 };
